/*===========================================================================

  File: DACIO_VC.cpp
    
  Description:

  This file contains the general C language DACIO routines and example demo

  This code example supports the DACIO I/O boards (http://www.tronisoft.com)
   
  As shipped this code is meant to compile under Windows.
  However, if you port the following functions the code 
  should work for nearly any standard C compiler:
   
  SerialOpenPort  - Open a com port (returns -1 for error)
  SerialClosePort - Close the com port 
  SerialWriteCh   - Write a byte to the COM port (returns 1 for success)
  SerialReadCh   - Read a byte from the COM port (returns byte or -1 for error)

  ===========================================================================*/


#include "stdafx.h"
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <windows.h>
#include "DACIO_VC.h"


/* Example demo program */
int main(int argc, char *argv[])
{
  int carry_on = 'y';
  if (argc<2)
  {
      fprintf(stderr,"Usage: dacio_vc port\n");
      return 1;
  }

  int x;

  while(carry_on == 'y' || carry_on == 'Y' || carry_on == 10)
  {
     //if(SerialOpenPort("COM4")<0)
     if(SerialOpenPortFast(argv[1])<0)
     {
	  printf("\nError opening COM port: %s", argv[1]);
         return 1;
     }

     printf("\n====================================\n");

     int MID = Get_ModuleID();
     printf("\nModule ID: %d\n", MID);

     int MVER = Get_ModuleSoftwareVersion();
     printf("Module Software Versions: %d\n", MVER);

      if(SetDigitalPORT('B', 0) == 0)
            printf("Setting PORTB I/O direction as all outputs\n");
      else
            printf("Failed to set I/O direction to PORTB\n");
      if(SetDigitalPORT('C', 0) == 0)
            printf("Setting PORTB I/O direction as all outputs\n");
      else
            printf("Failed to set I/O direction to PORTB\n");


      if(WriteDigitalPORT('B', 170) == 0);
      else printf("\nFailed to write 170 to PORTB\n");
      short_delay();

      if(WriteDigitalPORT('C', 170) == 0);
      else printf("\nFailed to write 170 to PORTC\n");
      short_delay();

      x = ReadDigitalPORT('B');
      if(x != 170)
         printf("\nError reading PORTB = %d. Should be 170\n", x);

      x = ReadDigitalPORT('C');
      if(x != 170)
         printf("\nError reading PORTC = %d. Should be 170\n", x);

      if(WriteDigitalPORT('B', 49) == 0);
      else printf("\nFailed to write 49 to PORTB\n");
      short_delay();

      if(WriteDigitalPORT('C', 49) == 0);
      else printf("\nFailed to write 49 to PORTC\n");
      short_delay();

      x = ReadDigitalPORT('B');
      if(x != 49)
         printf("\nError reading PORTB = %d. Should be 49\n", x);

      x = ReadDigitalPORT('C');
      if(x != 49)
         printf("\nError reading PORTC = %d. Should be 49\n", x);


      //==========================================================
      if(WriteDigitalPORT('B', 85) == 0);
      else printf("\nFailed to write 85 to PORTB\n");

      short_delay();

      if(WriteDigitalPORT('C', 85) == 0);
      else printf("\nFailed to write 85 to PORTC\n");
      short_delay();

      x = ReadDigitalPORT('B');
      if(x != 85)
      {
         printf("\nError reading PORTB = %d. Should be 85\n", x);
         Beep(400,1000);

      }
      x = ReadDigitalPORT('C');
      if(x != 85)
      {
         printf("\nError reading PORTC = %d. Should be 85\n", x);
         Beep(400,1000);
      }

      short_delay();


      x = Get_PORTA_Value(0);
      printf("\nAnalogue Channel A0 = %d\n", x);

      printf("\nPress To Test Comms Error - Red LED \n", x);
      getchar();

      printf("Closing port...\n");
      SerialClosePort();
      if(SerialOpenPortBad(argv[1])<0)
      {
	  printf("\nError opening COM port: %s", argv[1]);
          Beep(400,1000);
          return 1;
      }
      if(WriteDigitalPORT('C', 85) == 0);
      SerialClosePort();

      Beep(700,200);
      Sleep(100);
      Beep(700,200);
      printf("\nPress Enter To Test Again\n", x);
      carry_on = getchar();
      clrscr();
  }
  printf("Closing port...\n");
  SerialClosePort();
  getchar();
  return 0;
}


/* =======================================================================
                         DACIO specific functions 

   =====================================================================*/

int WriteDigitalPORT(char portch, int value)
{
  char temp_str[10];
  temp_str[0] = 0; 
  sprintf(temp_str, "!%c=%0.3d;", portch, value);   // e.g. "!B=255;" 
  SerialWriteString(temp_str);
  
  temp_str[0] = SerialReadCh();  /* should be '!" */
  temp_str[1] = SerialReadCh();  /* should be <CR> */
  if(temp_str[0] == '!' && temp_str[1] == 0x0D) return 0;
  else return -1;
}

int SetDigitalPORT(char portch, int value)
{
  char temp_str[10];
  temp_str[0] = 0; 
  sprintf(temp_str, "!S%c=%0.3d;", portch, value);   // e.g. "!SB=255;" 
  SerialWriteString(temp_str);
  
  temp_str[0] = SerialReadCh();  /* should be '!" */
  temp_str[1] = SerialReadCh();  /* should be <CR> */
  if(temp_str[0] == '!' && temp_str[1] == 0x0D) return 0;
  else return -1;
}

int ReadDigitalPORT(char portch)
{
  int result;
  char temp_str[10];
  temp_str[0] = 0; 
  sprintf(temp_str, "!%c?;", portch);   // e.g. "!B?;" 
  SerialWriteString(temp_str);

  temp_str[0] = SerialReadCh();  
  if(temp_str[0] == '!')             // got a good reply '!'.  
  {
  	 for(int i = 0; i < 4; i++)
   	 {
   	 	    temp_str[i] = SerialReadCh();  // get the 3 digits and get terminating (CR) char
	 } 
	 temp_str[i] = 0;               // discard terminating (CR) char 
     result = atoi(temp_str); // convert ASCII digits from unit into an integer
  }
  else
  {
      SerialReadCh();  // read '?';
   	  result = -1;
  }
  return result;
}

int Get_PORTA_Value(int ch)
{
  char temp_str[10];
  int result;
  SerialWriteCh('!'); SerialWriteCh('A'); SerialWriteCh(ch+'0'); SerialWriteCh('?'); SerialWriteCh(';');
  
  temp_str[0] = SerialReadCh();  
  if(temp_str[0] == '!')             // got a good reply '!'.  
  {
   	  for(int i = 0; i < 4; i++)
   	     temp_str[i] = SerialReadCh();  // get the 3 digits and get terminating (CR) char
	  temp_str[i] = 0;               // discard terminating (CR) char 
  }
  else
  {
      SerialReadCh();  // read '?';
	  return -1;
  }

  result = atoi(temp_str); // convert ASCII digits from unit into an integer
  return result;
}

int Get_ModuleID(void)
{
  char temp_str[10];
  int result;
  // send "!SMID?;"
  SerialWriteCh('!');
  SerialWriteCh('S');
  SerialWriteCh('M');
  SerialWriteCh('I');
  SerialWriteCh('D');
  SerialWriteCh('?');
  SerialWriteCh(';');

  temp_str[0] = SerialReadCh();
  if(temp_str[0] == '!')             // got a good reply '!'.
  {
   	  for(int i = 0; i < 4; i++)
   	     temp_str[i] = SerialReadCh();  // get the 3 digits and get terminating (CR) char
	  temp_str[i] = 0;                  // discard terminating (CR) char
  }
  else
  {
      SerialReadCh();  // read '?';
	  return -1;
  }

  result = atoi(temp_str); // convert ASCII digits from unit into an integer
  return result;
}

int Get_ModuleSoftwareVersion(void)
{
  char temp_str[10];
  int result;
  // send "!SMID?;"
  SerialWriteCh('!');
  SerialWriteCh('S');
  SerialWriteCh('V');
  SerialWriteCh('E');
  SerialWriteCh('R');
  SerialWriteCh('?');
  SerialWriteCh(';');

  temp_str[0] = SerialReadCh();
  if(temp_str[0] == '!')             // got a good reply '!'.
  {
   	  for(int i = 0; i < 3; i++)
   	     temp_str[i] = SerialReadCh();  // get the 2 digits and get terminating (CR) char
	  temp_str[i] = 0;                  // discard terminating (CR) char
  }
  else
  {
      SerialReadCh();  // read '?';
	  return -1;
  }

  result = atoi(temp_str); // convert ASCII digits from unit into an integer
  return result;
}


/* =======================================================================
                         Auxilliary functions 

   =====================================================================*/

void SerialWriteString(char *s)
{
	 while(*s)
	 {
        SerialWriteCh(*s);
        s++;
	 } 
}	

void pause()
{
  printf("Press ENTER\n");
  getchar();
}

void short_delay()
{
  Sleep(200);
}


